describe("Integration test", function () {
    var canvas, finished, callback, referenceImg,
        width = 204,
        height = 100;

    var fulfilled = function (value) {
        var defer = ayepromise.defer();
        defer.resolve(value);
        return defer.promise;
    };

    var loadDocFixture = function (url, callback) {
        var request = new window.XMLHttpRequest(),
            doc;

        request.onreadystatechange = function () {
            if (request.readyState === 4) {
                callback(request.responseXML);
            }
        };
        request.open('GET', url, true);
        // Seems to not work on Safari (https://developer.mozilla.org/en/HTML_in_XMLHttpRequest)
        request.responseType = "document";
        request.overrideMimeType("text/html");
        request.send(null);

        return doc;
    };

    beforeEach(function () {
        jasmine.addMatchers(diffHelper.matcher);

        canvas = $('<canvas width="' + width + '" height="' + height + '"></canvas>'); // Firefox adds a space between the divs and needs the canvas to fit horizontally for all content to be rendered

        referenceImg = $('<img src="'+ rasterizeHTMLTestHelper.fixturesPath + '/testResult.png" alt="test image"/>');

        finished = false;
        callback = jasmine.createSpy("callback").and.callFake(function () { finished = true; });
    });

    ifNotInWebkitIt("should take a document, inline all displayable content and render to the given canvas", function (done) {
        loadDocFixture(rasterizeHTMLTestHelper.fixturesPath + "test.html", function (doc) {
            rasterizeHTML.drawDocument(doc, canvas.get(0), {cache: 'none'}).then(function (result) {
                expect(result.errors).toEqual([]);
                expect(result.image).toEqualImage(referenceImg.get(0), 1);

                expect(canvas.get(0)).toEqualImage(referenceImg.get(0), 1);
                // expect(canvas.get(0)).toImageDiffEqual(referenceImg.get(0), 10);

                done();
            });
        });
    });

    ifNotInWebkitIt("should take a HTML string, inline all displayable content and render to the given canvas", function (done) {
        var html = rasterizeHTMLTestHelper.readHTMLFixture("test.html");

        rasterizeHTML.drawHTML(html, canvas.get(0), {baseUrl: rasterizeHTMLTestHelper.fixturesPath, cache: 'none'}).then(function (result) {
            expect(result.errors).toEqual([]);
            expect(result.image).toEqualImage(referenceImg.get(0), 1);

            expect(canvas.get(0)).toEqualImage(referenceImg.get(0), 1);
            // expect(canvas.get(0)).toImageDiffEqual(referenceImg.get(0), 70);

            done();
        });
    });

    ifNotInWebkitIt("should take a URL, inline all displayable content and render to the given canvas", function (done) {
        rasterizeHTML.drawURL(rasterizeHTMLTestHelper.fixturesPath + "testWithJs.html", canvas.get(0), {cache: 'none', executeJs: true}).then(function (result) {
            expect(result.errors).toEqual([]);
            expect(result.image).toEqualImage(referenceImg.get(0), 1);

            expect(canvas.get(0)).toEqualImage(referenceImg.get(0), 1);
            // expect(canvas.get(0)).toImageDiffEqual(referenceImg.get(0), 90);

            done();
        });
    });

    ifNotInWebkitIt("should take a URL, inline all displayable content and return the image", function (done) {
        rasterizeHTML.drawURL(rasterizeHTMLTestHelper.fixturesPath + "testWithJs.html", {cache: 'none', width: width, height: height, executeJs: true}).then(function (result) {
            expect(result.errors).toEqual([]);
            expect(result.image).toEqualImage(referenceImg.get(0), 1);
            // expect(result.image).toImageDiffEqual(referenceImg.get(0), 90);

            done();
        });
    });

    ifNotInPhantomJSAndNotLocalRunnerIt("should take a URL and load non UTF-8 content", function (done) {
        var inlineReferencesSpy = spyOn(rasterizeHTMLInline, 'inlineReferences').and.returnValue(fulfilled());

        rasterizeHTML.drawURL(rasterizeHTMLTestHelper.fixturesPath + "nonUTF8Encoding.html").then(function () {
            expect(inlineReferencesSpy).toHaveBeenCalled();

            var doc = inlineReferencesSpy.calls.mostRecent().args[0];

            // This fails if SpecRunner is opened locally in Firefox. Open over a local webserver helps here.
            expect(doc.body.innerHTML.trim()).toEqual('这是中文');

            done();
        });
    });
});
